package cn.ibizlab.r7.runtime.client.base.service;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import cn.ibizlab.r7.runtime.core.client.model.IDataEntity;
import cn.ibizlab.r7.runtime.client.base.config.BaseClientProperties;
import cn.ibizlab.r7.runtime.core.client.service.IDataEntityClientService;
import cn.ibizlab.r7.runtime.client.base.model.DataEntityImpl;
import cn.ibizlab.r7.runtime.client.base.feign.DataEntityFeignClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;
import cn.ibizlab.r7.runtime.util.feign.FeignRequestInterceptor;
import cn.ibizlab.r7.runtime.util.SearchContext;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import cn.ibizlab.r7.runtime.util.feign.suport.SearchContextFeignEncode;
import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.codec.Decoder;
import feign.codec.Encoder;

/**
 * 实体[DataEntity] 服务对象接口
 */
@Service
public class DataEntityClientServiceImpl implements IDataEntityClientService {

    DataEntityFeignClient dataEntityFeignClient;

    @Autowired
    public DataEntityClientServiceImpl(Decoder decoder, Encoder encoder, Client client, Contract contract ,FeignRequestInterceptor feignRequestInterceptor,BaseClientProperties baseClientProperties) {
        if (baseClientProperties.getServiceId()!=null) {
    		Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.dataEntityFeignClient = nameBuilder.target(DataEntityFeignClient.class,"http://"+baseClientProperties.getServiceId()+"/") ;
		}else if (baseClientProperties.getServiceUrl()!=null) {
			if (client instanceof LoadBalancerFeignClient) {
				client = ((LoadBalancerFeignClient) client).getDelegate();
			}
			Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.dataEntityFeignClient = nameBuilder.target(DataEntityFeignClient.class,"http://"+baseClientProperties.getServiceUrl()+"/") ;
		}
    }
    @Override
    public IDataEntity createModel() {
		return new DataEntityImpl();
	}


    public void remove(IDataEntity dataentity){
        dataEntityFeignClient.remove(dataentity.getDataEntityId()) ;
    }


    public void create(IDataEntity dataentity){
        IDataEntity clientModel = dataEntityFeignClient.create((DataEntityImpl)dataentity) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), dataentity.getClass(), false);
        copier.copy(clientModel, dataentity, null);
    }


    public void update(IDataEntity dataentity){
        IDataEntity clientModel = dataEntityFeignClient.update(dataentity.getDataEntityId(),(DataEntityImpl)dataentity) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), dataentity.getClass(), false);
        copier.copy(clientModel, dataentity, null);
    }


    public void initAll(IDataEntity dataentity){
        IDataEntity clientModel = dataEntityFeignClient.initAll(dataentity.getDataEntityId(),(DataEntityImpl)dataentity) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), dataentity.getClass(), false);
        copier.copy(clientModel, dataentity, null);
    }


    public void getDraft(IDataEntity dataentity){
        IDataEntity clientModel = dataEntityFeignClient.getDraft(dataentity.getDataEntityId(),(DataEntityImpl)dataentity) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), dataentity.getClass(), false);
        copier.copy(clientModel, dataentity, null);
    }


    public void get(IDataEntity dataentity){
        IDataEntity clientModel = dataEntityFeignClient.get(dataentity.getDataEntityId()) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), dataentity.getClass(), false);
        copier.copy(clientModel, dataentity, null);
    }


    public void save(IDataEntity dataentity){
        IDataEntity clientModel = dataEntityFeignClient.save(dataentity.getDataEntityId(),(DataEntityImpl)dataentity) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), dataentity.getClass(), false);
        copier.copy(clientModel, dataentity, null);
    }


    public void checkKey(IDataEntity dataentity){
        IDataEntity clientModel = dataEntityFeignClient.checkKey(dataentity.getDataEntityId(),(DataEntityImpl)dataentity) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), dataentity.getClass(), false);
        copier.copy(clientModel, dataentity, null);
    }


    public Page<IDataEntity> fetchSelectDEAll(SearchContext context){
        Page<DataEntityImpl> page = this.dataEntityFeignClient.fetchSelectDEAll(context) ;
        return new PageImpl(page.getContent(),context.getPageable(),page.getTotalElements());
    }


    public Page<IDataEntity> fetchDefault(SearchContext context){
        Page<DataEntityImpl> page = this.dataEntityFeignClient.fetchDefault(context) ;
        return new PageImpl(page.getContent(),context.getPageable(),page.getTotalElements());
    }



}

