package cn.ibizlab.r7.runtime.client.base.service;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import cn.ibizlab.r7.runtime.core.client.model.ILoginAccountPwd;
import cn.ibizlab.r7.runtime.client.base.config.BaseClientProperties;
import cn.ibizlab.r7.runtime.core.client.service.ILoginAccountPwdClientService;
import cn.ibizlab.r7.runtime.client.base.model.LoginAccountPwdImpl;
import cn.ibizlab.r7.runtime.client.base.feign.LoginAccountPwdFeignClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;
import cn.ibizlab.r7.runtime.util.feign.FeignRequestInterceptor;
import cn.ibizlab.r7.runtime.util.SearchContext;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import cn.ibizlab.r7.runtime.util.feign.suport.SearchContextFeignEncode;
import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.codec.Decoder;
import feign.codec.Encoder;

/**
 * 实体[LoginAccountPwd] 服务对象接口
 */
@Service
public class LoginAccountPwdClientServiceImpl implements ILoginAccountPwdClientService {

    LoginAccountPwdFeignClient loginAccountPwdFeignClient;

    @Autowired
    public LoginAccountPwdClientServiceImpl(Decoder decoder, Encoder encoder, Client client, Contract contract ,FeignRequestInterceptor feignRequestInterceptor,BaseClientProperties baseClientProperties) {
        if (baseClientProperties.getServiceId()!=null) {
    		Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.loginAccountPwdFeignClient = nameBuilder.target(LoginAccountPwdFeignClient.class,"http://"+baseClientProperties.getServiceId()+"/") ;
		}else if (baseClientProperties.getServiceUrl()!=null) {
			if (client instanceof LoadBalancerFeignClient) {
				client = ((LoadBalancerFeignClient) client).getDelegate();
			}
			Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.loginAccountPwdFeignClient = nameBuilder.target(LoginAccountPwdFeignClient.class,"http://"+baseClientProperties.getServiceUrl()+"/") ;
		}
    }
    @Override
    public ILoginAccountPwd createModel() {
		return new LoginAccountPwdImpl();
	}


    public void checkKey(ILoginAccountPwd loginaccountpwd){
        ILoginAccountPwd clientModel = loginAccountPwdFeignClient.checkKey(loginaccountpwd.getLoginAccountPwdId(),(LoginAccountPwdImpl)loginaccountpwd) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), loginaccountpwd.getClass(), false);
        copier.copy(clientModel, loginaccountpwd, null);
    }


    public void update(ILoginAccountPwd loginaccountpwd){
        ILoginAccountPwd clientModel = loginAccountPwdFeignClient.update(loginaccountpwd.getLoginAccountPwdId(),(LoginAccountPwdImpl)loginaccountpwd) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), loginaccountpwd.getClass(), false);
        copier.copy(clientModel, loginaccountpwd, null);
    }


    public void remove(ILoginAccountPwd loginaccountpwd){
        loginAccountPwdFeignClient.remove(loginaccountpwd.getLoginAccountPwdId()) ;
    }


    public void get(ILoginAccountPwd loginaccountpwd){
        ILoginAccountPwd clientModel = loginAccountPwdFeignClient.get(loginaccountpwd.getLoginAccountPwdId()) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), loginaccountpwd.getClass(), false);
        copier.copy(clientModel, loginaccountpwd, null);
    }


    public void save(ILoginAccountPwd loginaccountpwd){
        ILoginAccountPwd clientModel = loginAccountPwdFeignClient.save(loginaccountpwd.getLoginAccountPwdId(),(LoginAccountPwdImpl)loginaccountpwd) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), loginaccountpwd.getClass(), false);
        copier.copy(clientModel, loginaccountpwd, null);
    }


    public void create(ILoginAccountPwd loginaccountpwd){
        ILoginAccountPwd clientModel = loginAccountPwdFeignClient.create((LoginAccountPwdImpl)loginaccountpwd) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), loginaccountpwd.getClass(), false);
        copier.copy(clientModel, loginaccountpwd, null);
    }


    public void getDraft(ILoginAccountPwd loginaccountpwd){
        ILoginAccountPwd clientModel = loginAccountPwdFeignClient.getDraft(loginaccountpwd.getLoginAccountPwdId(),(LoginAccountPwdImpl)loginaccountpwd) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), loginaccountpwd.getClass(), false);
        copier.copy(clientModel, loginaccountpwd, null);
    }


    public Page<ILoginAccountPwd> fetchDefault(SearchContext context){
        Page<LoginAccountPwdImpl> page = this.loginAccountPwdFeignClient.fetchDefault(context) ;
        return new PageImpl(page.getContent(),context.getPageable(),page.getTotalElements());
    }



}

