package cn.ibizlab.r7.runtime.client.base.service;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import cn.ibizlab.r7.runtime.core.client.model.ISystem;
import cn.ibizlab.r7.runtime.client.base.config.BaseClientProperties;
import cn.ibizlab.r7.runtime.core.client.service.ISystemClientService;
import cn.ibizlab.r7.runtime.client.base.model.SystemImpl;
import cn.ibizlab.r7.runtime.client.base.feign.SystemFeignClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;
import cn.ibizlab.r7.runtime.util.feign.FeignRequestInterceptor;
import cn.ibizlab.r7.runtime.util.SearchContext;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import cn.ibizlab.r7.runtime.util.feign.suport.SearchContextFeignEncode;
import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.codec.Decoder;
import feign.codec.Encoder;

/**
 * 实体[System] 服务对象接口
 */
@Service
public class SystemClientServiceImpl implements ISystemClientService {

    SystemFeignClient systemFeignClient;

    @Autowired
    public SystemClientServiceImpl(Decoder decoder, Encoder encoder, Client client, Contract contract ,FeignRequestInterceptor feignRequestInterceptor,BaseClientProperties baseClientProperties) {
        if (baseClientProperties.getServiceId()!=null) {
    		Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.systemFeignClient = nameBuilder.target(SystemFeignClient.class,"http://"+baseClientProperties.getServiceId()+"/") ;
		}else if (baseClientProperties.getServiceUrl()!=null) {
			if (client instanceof LoadBalancerFeignClient) {
				client = ((LoadBalancerFeignClient) client).getDelegate();
			}
			Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.systemFeignClient = nameBuilder.target(SystemFeignClient.class,"http://"+baseClientProperties.getServiceUrl()+"/") ;
		}
    }
    @Override
    public ISystem createModel() {
		return new SystemImpl();
	}


    public void create(ISystem system){
        ISystem clientModel = systemFeignClient.create((SystemImpl)system) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), system.getClass(), false);
        copier.copy(clientModel, system, null);
    }


    public void update(ISystem system){
        ISystem clientModel = systemFeignClient.update(system.getSystemId(),(SystemImpl)system) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), system.getClass(), false);
        copier.copy(clientModel, system, null);
    }


    public void get(ISystem system){
        ISystem clientModel = systemFeignClient.get(system.getSystemId()) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), system.getClass(), false);
        copier.copy(clientModel, system, null);
    }


    public void remove(ISystem system){
        systemFeignClient.remove(system.getSystemId()) ;
    }


    public void save(ISystem system){
        ISystem clientModel = systemFeignClient.save(system.getSystemId(),(SystemImpl)system) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), system.getClass(), false);
        copier.copy(clientModel, system, null);
    }


    public void checkKey(ISystem system){
        ISystem clientModel = systemFeignClient.checkKey(system.getSystemId(),(SystemImpl)system) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), system.getClass(), false);
        copier.copy(clientModel, system, null);
    }


    public void getDraft(ISystem system){
        ISystem clientModel = systemFeignClient.getDraft(system.getSystemId(),(SystemImpl)system) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), system.getClass(), false);
        copier.copy(clientModel, system, null);
    }


    public Page<ISystem> fetchDefault(SearchContext context){
        Page<SystemImpl> page = this.systemFeignClient.fetchDefault(context) ;
        return new PageImpl(page.getContent(),context.getPageable(),page.getTotalElements());
    }



}

