package cn.ibizlab.r7.runtime.client.base.service;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import cn.ibizlab.r7.runtime.core.client.model.IUserGroup;
import cn.ibizlab.r7.runtime.client.base.config.BaseClientProperties;
import cn.ibizlab.r7.runtime.core.client.service.IUserGroupClientService;
import cn.ibizlab.r7.runtime.client.base.model.UserGroupImpl;
import cn.ibizlab.r7.runtime.client.base.feign.UserGroupFeignClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;
import cn.ibizlab.r7.runtime.util.feign.FeignRequestInterceptor;
import cn.ibizlab.r7.runtime.util.SearchContext;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import cn.ibizlab.r7.runtime.util.feign.suport.SearchContextFeignEncode;
import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.codec.Decoder;
import feign.codec.Encoder;

/**
 * 实体[UserGroup] 服务对象接口
 */
@Service
public class UserGroupClientServiceImpl implements IUserGroupClientService {

    UserGroupFeignClient userGroupFeignClient;

    @Autowired
    public UserGroupClientServiceImpl(Decoder decoder, Encoder encoder, Client client, Contract contract ,FeignRequestInterceptor feignRequestInterceptor,BaseClientProperties baseClientProperties) {
        if (baseClientProperties.getServiceId()!=null) {
    		Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userGroupFeignClient = nameBuilder.target(UserGroupFeignClient.class,"http://"+baseClientProperties.getServiceId()+"/") ;
		}else if (baseClientProperties.getServiceUrl()!=null) {
			if (client instanceof LoadBalancerFeignClient) {
				client = ((LoadBalancerFeignClient) client).getDelegate();
			}
			Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userGroupFeignClient = nameBuilder.target(UserGroupFeignClient.class,"http://"+baseClientProperties.getServiceUrl()+"/") ;
		}
    }
    @Override
    public IUserGroup createModel() {
		return new UserGroupImpl();
	}


    public void create(IUserGroup usergroup){
        IUserGroup clientModel = userGroupFeignClient.create((UserGroupImpl)usergroup) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), usergroup.getClass(), false);
        copier.copy(clientModel, usergroup, null);
    }


    public void save(IUserGroup usergroup){
        IUserGroup clientModel = userGroupFeignClient.save(usergroup.getUserGroupId(),(UserGroupImpl)usergroup) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), usergroup.getClass(), false);
        copier.copy(clientModel, usergroup, null);
    }


    public void checkKey(IUserGroup usergroup){
        IUserGroup clientModel = userGroupFeignClient.checkKey(usergroup.getUserGroupId(),(UserGroupImpl)usergroup) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), usergroup.getClass(), false);
        copier.copy(clientModel, usergroup, null);
    }


    public void getDraft(IUserGroup usergroup){
        IUserGroup clientModel = userGroupFeignClient.getDraft(usergroup.getUserGroupId(),(UserGroupImpl)usergroup) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), usergroup.getClass(), false);
        copier.copy(clientModel, usergroup, null);
    }


    public void get(IUserGroup usergroup){
        IUserGroup clientModel = userGroupFeignClient.get(usergroup.getUserGroupId()) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), usergroup.getClass(), false);
        copier.copy(clientModel, usergroup, null);
    }


    public void update(IUserGroup usergroup){
        IUserGroup clientModel = userGroupFeignClient.update(usergroup.getUserGroupId(),(UserGroupImpl)usergroup) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), usergroup.getClass(), false);
        copier.copy(clientModel, usergroup, null);
    }


    public void remove(IUserGroup usergroup){
        userGroupFeignClient.remove(usergroup.getUserGroupId()) ;
    }


    public Page<IUserGroup> fetchDefault(SearchContext context){
        Page<UserGroupImpl> page = this.userGroupFeignClient.fetchDefault(context) ;
        return new PageImpl(page.getContent(),context.getPageable(),page.getTotalElements());
    }



}

