package cn.ibizlab.r7.runtime.client.base.service;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import cn.ibizlab.r7.runtime.core.client.model.IUserGroupDetail;
import cn.ibizlab.r7.runtime.client.base.config.BaseClientProperties;
import cn.ibizlab.r7.runtime.core.client.service.IUserGroupDetailClientService;
import cn.ibizlab.r7.runtime.client.base.model.UserGroupDetailImpl;
import cn.ibizlab.r7.runtime.client.base.feign.UserGroupDetailFeignClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;
import cn.ibizlab.r7.runtime.util.feign.FeignRequestInterceptor;
import cn.ibizlab.r7.runtime.util.SearchContext;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import cn.ibizlab.r7.runtime.util.feign.suport.SearchContextFeignEncode;
import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.codec.Decoder;
import feign.codec.Encoder;

/**
 * 实体[UserGroupDetail] 服务对象接口
 */
@Service
public class UserGroupDetailClientServiceImpl implements IUserGroupDetailClientService {

    UserGroupDetailFeignClient userGroupDetailFeignClient;

    @Autowired
    public UserGroupDetailClientServiceImpl(Decoder decoder, Encoder encoder, Client client, Contract contract ,FeignRequestInterceptor feignRequestInterceptor,BaseClientProperties baseClientProperties) {
        if (baseClientProperties.getServiceId()!=null) {
    		Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userGroupDetailFeignClient = nameBuilder.target(UserGroupDetailFeignClient.class,"http://"+baseClientProperties.getServiceId()+"/") ;
		}else if (baseClientProperties.getServiceUrl()!=null) {
			if (client instanceof LoadBalancerFeignClient) {
				client = ((LoadBalancerFeignClient) client).getDelegate();
			}
			Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userGroupDetailFeignClient = nameBuilder.target(UserGroupDetailFeignClient.class,"http://"+baseClientProperties.getServiceUrl()+"/") ;
		}
    }
    @Override
    public IUserGroupDetail createModel() {
		return new UserGroupDetailImpl();
	}


    public void get(IUserGroupDetail usergroupdetail){
        IUserGroupDetail clientModel = userGroupDetailFeignClient.get(usergroupdetail.getUserGroupDetailId()) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), usergroupdetail.getClass(), false);
        copier.copy(clientModel, usergroupdetail, null);
    }


    public void getDraft(IUserGroupDetail usergroupdetail){
        IUserGroupDetail clientModel = userGroupDetailFeignClient.getDraft(usergroupdetail.getUserGroupDetailId(),(UserGroupDetailImpl)usergroupdetail) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), usergroupdetail.getClass(), false);
        copier.copy(clientModel, usergroupdetail, null);
    }


    public void create(IUserGroupDetail usergroupdetail){
        IUserGroupDetail clientModel = userGroupDetailFeignClient.create((UserGroupDetailImpl)usergroupdetail) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), usergroupdetail.getClass(), false);
        copier.copy(clientModel, usergroupdetail, null);
    }


    public void save(IUserGroupDetail usergroupdetail){
        IUserGroupDetail clientModel = userGroupDetailFeignClient.save(usergroupdetail.getUserGroupDetailId(),(UserGroupDetailImpl)usergroupdetail) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), usergroupdetail.getClass(), false);
        copier.copy(clientModel, usergroupdetail, null);
    }


    public void checkKey(IUserGroupDetail usergroupdetail){
        IUserGroupDetail clientModel = userGroupDetailFeignClient.checkKey(usergroupdetail.getUserGroupDetailId(),(UserGroupDetailImpl)usergroupdetail) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), usergroupdetail.getClass(), false);
        copier.copy(clientModel, usergroupdetail, null);
    }


    public void update(IUserGroupDetail usergroupdetail){
        IUserGroupDetail clientModel = userGroupDetailFeignClient.update(usergroupdetail.getUserGroupDetailId(),(UserGroupDetailImpl)usergroupdetail) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), usergroupdetail.getClass(), false);
        copier.copy(clientModel, usergroupdetail, null);
    }


    public void remove(IUserGroupDetail usergroupdetail){
        userGroupDetailFeignClient.remove(usergroupdetail.getUserGroupDetailId()) ;
    }


    public Page<IUserGroupDetail> fetchDefault(SearchContext context){
        Page<UserGroupDetailImpl> page = this.userGroupDetailFeignClient.fetchDefault(context) ;
        return new PageImpl(page.getContent(),context.getPageable(),page.getTotalElements());
    }



}

