package cn.ibizlab.r7.runtime.client.base.service;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import cn.ibizlab.r7.runtime.core.client.model.IUserObject;
import cn.ibizlab.r7.runtime.client.base.config.BaseClientProperties;
import cn.ibizlab.r7.runtime.core.client.service.IUserObjectClientService;
import cn.ibizlab.r7.runtime.client.base.model.UserObjectImpl;
import cn.ibizlab.r7.runtime.client.base.feign.UserObjectFeignClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;
import cn.ibizlab.r7.runtime.util.feign.FeignRequestInterceptor;
import cn.ibizlab.r7.runtime.util.SearchContext;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import cn.ibizlab.r7.runtime.util.feign.suport.SearchContextFeignEncode;
import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.codec.Decoder;
import feign.codec.Encoder;

/**
 * 实体[UserObject] 服务对象接口
 */
@Service
public class UserObjectClientServiceImpl implements IUserObjectClientService {

    UserObjectFeignClient userObjectFeignClient;

    @Autowired
    public UserObjectClientServiceImpl(Decoder decoder, Encoder encoder, Client client, Contract contract ,FeignRequestInterceptor feignRequestInterceptor,BaseClientProperties baseClientProperties) {
        if (baseClientProperties.getServiceId()!=null) {
    		Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userObjectFeignClient = nameBuilder.target(UserObjectFeignClient.class,"http://"+baseClientProperties.getServiceId()+"/") ;
		}else if (baseClientProperties.getServiceUrl()!=null) {
			if (client instanceof LoadBalancerFeignClient) {
				client = ((LoadBalancerFeignClient) client).getDelegate();
			}
			Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userObjectFeignClient = nameBuilder.target(UserObjectFeignClient.class,"http://"+baseClientProperties.getServiceUrl()+"/") ;
		}
    }
    @Override
    public IUserObject createModel() {
		return new UserObjectImpl();
	}


    public void checkKey(IUserObject userobject){
        IUserObject clientModel = userObjectFeignClient.checkKey(userobject.getUserObjectId(),(UserObjectImpl)userobject) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userobject.getClass(), false);
        copier.copy(clientModel, userobject, null);
    }


    public void save(IUserObject userobject){
        IUserObject clientModel = userObjectFeignClient.save(userobject.getUserObjectId(),(UserObjectImpl)userobject) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userobject.getClass(), false);
        copier.copy(clientModel, userobject, null);
    }


    public void create(IUserObject userobject){
        IUserObject clientModel = userObjectFeignClient.create((UserObjectImpl)userobject) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userobject.getClass(), false);
        copier.copy(clientModel, userobject, null);
    }


    public void getDraft(IUserObject userobject){
        IUserObject clientModel = userObjectFeignClient.getDraft(userobject.getUserObjectId(),(UserObjectImpl)userobject) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userobject.getClass(), false);
        copier.copy(clientModel, userobject, null);
    }


    public void remove(IUserObject userobject){
        userObjectFeignClient.remove(userobject.getUserObjectId()) ;
    }


    public void get(IUserObject userobject){
        IUserObject clientModel = userObjectFeignClient.get(userobject.getUserObjectId()) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userobject.getClass(), false);
        copier.copy(clientModel, userobject, null);
    }


    public void update(IUserObject userobject){
        IUserObject clientModel = userObjectFeignClient.update(userobject.getUserObjectId(),(UserObjectImpl)userobject) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userobject.getClass(), false);
        copier.copy(clientModel, userobject, null);
    }


    public Page<IUserObject> fetchDefault(SearchContext context){
        Page<UserObjectImpl> page = this.userObjectFeignClient.fetchDefault(context) ;
        return new PageImpl(page.getContent(),context.getPageable(),page.getTotalElements());
    }



}

