package cn.ibizlab.r7.runtime.client.base.service;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import cn.ibizlab.r7.runtime.core.client.model.IUserRoleDataAction;
import cn.ibizlab.r7.runtime.client.base.config.BaseClientProperties;
import cn.ibizlab.r7.runtime.core.client.service.IUserRoleDataActionClientService;
import cn.ibizlab.r7.runtime.client.base.model.UserRoleDataActionImpl;
import cn.ibizlab.r7.runtime.client.base.feign.UserRoleDataActionFeignClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;
import cn.ibizlab.r7.runtime.util.feign.FeignRequestInterceptor;
import cn.ibizlab.r7.runtime.util.SearchContext;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import cn.ibizlab.r7.runtime.util.feign.suport.SearchContextFeignEncode;
import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.codec.Decoder;
import feign.codec.Encoder;

/**
 * 实体[UserRoleDataAction] 服务对象接口
 */
@Service
public class UserRoleDataActionClientServiceImpl implements IUserRoleDataActionClientService {

    UserRoleDataActionFeignClient userRoleDataActionFeignClient;

    @Autowired
    public UserRoleDataActionClientServiceImpl(Decoder decoder, Encoder encoder, Client client, Contract contract ,FeignRequestInterceptor feignRequestInterceptor,BaseClientProperties baseClientProperties) {
        if (baseClientProperties.getServiceId()!=null) {
    		Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userRoleDataActionFeignClient = nameBuilder.target(UserRoleDataActionFeignClient.class,"http://"+baseClientProperties.getServiceId()+"/") ;
		}else if (baseClientProperties.getServiceUrl()!=null) {
			if (client instanceof LoadBalancerFeignClient) {
				client = ((LoadBalancerFeignClient) client).getDelegate();
			}
			Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userRoleDataActionFeignClient = nameBuilder.target(UserRoleDataActionFeignClient.class,"http://"+baseClientProperties.getServiceUrl()+"/") ;
		}
    }
    @Override
    public IUserRoleDataAction createModel() {
		return new UserRoleDataActionImpl();
	}


    public void save(IUserRoleDataAction userroledataaction){
        IUserRoleDataAction clientModel = userRoleDataActionFeignClient.save(userroledataaction.getUserRoleDataActionId(),(UserRoleDataActionImpl)userroledataaction) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledataaction.getClass(), false);
        copier.copy(clientModel, userroledataaction, null);
    }


    public void create(IUserRoleDataAction userroledataaction){
        IUserRoleDataAction clientModel = userRoleDataActionFeignClient.create((UserRoleDataActionImpl)userroledataaction) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledataaction.getClass(), false);
        copier.copy(clientModel, userroledataaction, null);
    }


    public void update(IUserRoleDataAction userroledataaction){
        IUserRoleDataAction clientModel = userRoleDataActionFeignClient.update(userroledataaction.getUserRoleDataActionId(),(UserRoleDataActionImpl)userroledataaction) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledataaction.getClass(), false);
        copier.copy(clientModel, userroledataaction, null);
    }


    public void getDraft(IUserRoleDataAction userroledataaction){
        IUserRoleDataAction clientModel = userRoleDataActionFeignClient.getDraft(userroledataaction.getUserRoleDataActionId(),(UserRoleDataActionImpl)userroledataaction) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledataaction.getClass(), false);
        copier.copy(clientModel, userroledataaction, null);
    }


    public void get(IUserRoleDataAction userroledataaction){
        IUserRoleDataAction clientModel = userRoleDataActionFeignClient.get(userroledataaction.getUserRoleDataActionId()) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledataaction.getClass(), false);
        copier.copy(clientModel, userroledataaction, null);
    }


    public void checkKey(IUserRoleDataAction userroledataaction){
        IUserRoleDataAction clientModel = userRoleDataActionFeignClient.checkKey(userroledataaction.getUserRoleDataActionId(),(UserRoleDataActionImpl)userroledataaction) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledataaction.getClass(), false);
        copier.copy(clientModel, userroledataaction, null);
    }


    public void remove(IUserRoleDataAction userroledataaction){
        userRoleDataActionFeignClient.remove(userroledataaction.getUserRoleDataActionId()) ;
    }


    public Page<IUserRoleDataAction> fetchDefault(SearchContext context){
        Page<UserRoleDataActionImpl> page = this.userRoleDataActionFeignClient.fetchDefault(context) ;
        return new PageImpl(page.getContent(),context.getPageable(),page.getTotalElements());
    }



}

