package cn.ibizlab.r7.runtime.client.base.service;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import cn.ibizlab.r7.runtime.core.client.model.IUserRoleDatas;
import cn.ibizlab.r7.runtime.client.base.config.BaseClientProperties;
import cn.ibizlab.r7.runtime.core.client.service.IUserRoleDatasClientService;
import cn.ibizlab.r7.runtime.client.base.model.UserRoleDatasImpl;
import cn.ibizlab.r7.runtime.client.base.feign.UserRoleDatasFeignClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;
import cn.ibizlab.r7.runtime.util.feign.FeignRequestInterceptor;
import cn.ibizlab.r7.runtime.util.SearchContext;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import cn.ibizlab.r7.runtime.util.feign.suport.SearchContextFeignEncode;
import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.codec.Decoder;
import feign.codec.Encoder;

/**
 * 实体[UserRoleDatas] 服务对象接口
 */
@Service
public class UserRoleDatasClientServiceImpl implements IUserRoleDatasClientService {

    UserRoleDatasFeignClient userRoleDatasFeignClient;

    @Autowired
    public UserRoleDatasClientServiceImpl(Decoder decoder, Encoder encoder, Client client, Contract contract ,FeignRequestInterceptor feignRequestInterceptor,BaseClientProperties baseClientProperties) {
        if (baseClientProperties.getServiceId()!=null) {
    		Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userRoleDatasFeignClient = nameBuilder.target(UserRoleDatasFeignClient.class,"http://"+baseClientProperties.getServiceId()+"/") ;
		}else if (baseClientProperties.getServiceUrl()!=null) {
			if (client instanceof LoadBalancerFeignClient) {
				client = ((LoadBalancerFeignClient) client).getDelegate();
			}
			Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userRoleDatasFeignClient = nameBuilder.target(UserRoleDatasFeignClient.class,"http://"+baseClientProperties.getServiceUrl()+"/") ;
		}
    }
    @Override
    public IUserRoleDatas createModel() {
		return new UserRoleDatasImpl();
	}


    public void save(IUserRoleDatas userroledatas){
        IUserRoleDatas clientModel = userRoleDatasFeignClient.save(userroledatas.getUserRoleDatasId(),(UserRoleDatasImpl)userroledatas) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledatas.getClass(), false);
        copier.copy(clientModel, userroledatas, null);
    }


    public void create(IUserRoleDatas userroledatas){
        IUserRoleDatas clientModel = userRoleDatasFeignClient.create((UserRoleDatasImpl)userroledatas) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledatas.getClass(), false);
        copier.copy(clientModel, userroledatas, null);
    }


    public void update(IUserRoleDatas userroledatas){
        IUserRoleDatas clientModel = userRoleDatasFeignClient.update(userroledatas.getUserRoleDatasId(),(UserRoleDatasImpl)userroledatas) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledatas.getClass(), false);
        copier.copy(clientModel, userroledatas, null);
    }


    public void checkKey(IUserRoleDatas userroledatas){
        IUserRoleDatas clientModel = userRoleDatasFeignClient.checkKey(userroledatas.getUserRoleDatasId(),(UserRoleDatasImpl)userroledatas) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledatas.getClass(), false);
        copier.copy(clientModel, userroledatas, null);
    }


    public void get(IUserRoleDatas userroledatas){
        IUserRoleDatas clientModel = userRoleDatasFeignClient.get(userroledatas.getUserRoleDatasId()) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledatas.getClass(), false);
        copier.copy(clientModel, userroledatas, null);
    }


    public void getDraft(IUserRoleDatas userroledatas){
        IUserRoleDatas clientModel = userRoleDatasFeignClient.getDraft(userroledatas.getUserRoleDatasId(),(UserRoleDatasImpl)userroledatas) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledatas.getClass(), false);
        copier.copy(clientModel, userroledatas, null);
    }


    public void remove(IUserRoleDatas userroledatas){
        userRoleDatasFeignClient.remove(userroledatas.getUserRoleDatasId()) ;
    }


    public Page<IUserRoleDatas> fetchDefault(SearchContext context){
        Page<UserRoleDatasImpl> page = this.userRoleDatasFeignClient.fetchDefault(context) ;
        return new PageImpl(page.getContent(),context.getPageable(),page.getTotalElements());
    }



}

