package cn.ibizlab.r7.runtime.client.base.service;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import cn.ibizlab.r7.runtime.core.client.model.IUserRoleDetail;
import cn.ibizlab.r7.runtime.client.base.config.BaseClientProperties;
import cn.ibizlab.r7.runtime.core.client.service.IUserRoleDetailClientService;
import cn.ibizlab.r7.runtime.client.base.model.UserRoleDetailImpl;
import cn.ibizlab.r7.runtime.client.base.feign.UserRoleDetailFeignClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;
import cn.ibizlab.r7.runtime.util.feign.FeignRequestInterceptor;
import cn.ibizlab.r7.runtime.util.SearchContext;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import cn.ibizlab.r7.runtime.util.feign.suport.SearchContextFeignEncode;
import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.codec.Decoder;
import feign.codec.Encoder;

/**
 * 实体[UserRoleDetail] 服务对象接口
 */
@Service
public class UserRoleDetailClientServiceImpl implements IUserRoleDetailClientService {

    UserRoleDetailFeignClient userRoleDetailFeignClient;

    @Autowired
    public UserRoleDetailClientServiceImpl(Decoder decoder, Encoder encoder, Client client, Contract contract ,FeignRequestInterceptor feignRequestInterceptor,BaseClientProperties baseClientProperties) {
        if (baseClientProperties.getServiceId()!=null) {
    		Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userRoleDetailFeignClient = nameBuilder.target(UserRoleDetailFeignClient.class,"http://"+baseClientProperties.getServiceId()+"/") ;
		}else if (baseClientProperties.getServiceUrl()!=null) {
			if (client instanceof LoadBalancerFeignClient) {
				client = ((LoadBalancerFeignClient) client).getDelegate();
			}
			Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userRoleDetailFeignClient = nameBuilder.target(UserRoleDetailFeignClient.class,"http://"+baseClientProperties.getServiceUrl()+"/") ;
		}
    }
    @Override
    public IUserRoleDetail createModel() {
		return new UserRoleDetailImpl();
	}


    public void remove(IUserRoleDetail userroledetail){
        userRoleDetailFeignClient.remove(userroledetail.getUserRoleDetailId()) ;
    }


    public void update(IUserRoleDetail userroledetail){
        IUserRoleDetail clientModel = userRoleDetailFeignClient.update(userroledetail.getUserRoleDetailId(),(UserRoleDetailImpl)userroledetail) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledetail.getClass(), false);
        copier.copy(clientModel, userroledetail, null);
    }


    public void save(IUserRoleDetail userroledetail){
        IUserRoleDetail clientModel = userRoleDetailFeignClient.save(userroledetail.getUserRoleDetailId(),(UserRoleDetailImpl)userroledetail) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledetail.getClass(), false);
        copier.copy(clientModel, userroledetail, null);
    }


    public void create(IUserRoleDetail userroledetail){
        IUserRoleDetail clientModel = userRoleDetailFeignClient.create((UserRoleDetailImpl)userroledetail) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledetail.getClass(), false);
        copier.copy(clientModel, userroledetail, null);
    }


    public void getDraft(IUserRoleDetail userroledetail){
        IUserRoleDetail clientModel = userRoleDetailFeignClient.getDraft(userroledetail.getUserRoleDetailId(),(UserRoleDetailImpl)userroledetail) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledetail.getClass(), false);
        copier.copy(clientModel, userroledetail, null);
    }


    public void checkKey(IUserRoleDetail userroledetail){
        IUserRoleDetail clientModel = userRoleDetailFeignClient.checkKey(userroledetail.getUserRoleDetailId(),(UserRoleDetailImpl)userroledetail) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledetail.getClass(), false);
        copier.copy(clientModel, userroledetail, null);
    }


    public void get(IUserRoleDetail userroledetail){
        IUserRoleDetail clientModel = userRoleDetailFeignClient.get(userroledetail.getUserRoleDetailId()) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroledetail.getClass(), false);
        copier.copy(clientModel, userroledetail, null);
    }


    public Page<IUserRoleDetail> fetchDefault(SearchContext context){
        Page<UserRoleDetailImpl> page = this.userRoleDetailFeignClient.fetchDefault(context) ;
        return new PageImpl(page.getContent(),context.getPageable(),page.getTotalElements());
    }



}

