package cn.ibizlab.r7.runtime.client.base.service;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import cn.ibizlab.r7.runtime.core.client.model.IUserRoleRes;
import cn.ibizlab.r7.runtime.client.base.config.BaseClientProperties;
import cn.ibizlab.r7.runtime.core.client.service.IUserRoleResClientService;
import cn.ibizlab.r7.runtime.client.base.model.UserRoleResImpl;
import cn.ibizlab.r7.runtime.client.base.feign.UserRoleResFeignClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;
import cn.ibizlab.r7.runtime.util.feign.FeignRequestInterceptor;
import cn.ibizlab.r7.runtime.util.SearchContext;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import cn.ibizlab.r7.runtime.util.feign.suport.SearchContextFeignEncode;
import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.codec.Decoder;
import feign.codec.Encoder;

/**
 * 实体[UserRoleRes] 服务对象接口
 */
@Service
public class UserRoleResClientServiceImpl implements IUserRoleResClientService {

    UserRoleResFeignClient userRoleResFeignClient;

    @Autowired
    public UserRoleResClientServiceImpl(Decoder decoder, Encoder encoder, Client client, Contract contract ,FeignRequestInterceptor feignRequestInterceptor,BaseClientProperties baseClientProperties) {
        if (baseClientProperties.getServiceId()!=null) {
    		Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userRoleResFeignClient = nameBuilder.target(UserRoleResFeignClient.class,"http://"+baseClientProperties.getServiceId()+"/") ;
		}else if (baseClientProperties.getServiceUrl()!=null) {
			if (client instanceof LoadBalancerFeignClient) {
				client = ((LoadBalancerFeignClient) client).getDelegate();
			}
			Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userRoleResFeignClient = nameBuilder.target(UserRoleResFeignClient.class,"http://"+baseClientProperties.getServiceUrl()+"/") ;
		}
    }
    @Override
    public IUserRoleRes createModel() {
		return new UserRoleResImpl();
	}


    public void getDraft(IUserRoleRes userroleres){
        IUserRoleRes clientModel = userRoleResFeignClient.getDraft(userroleres.getUserRoleResId(),(UserRoleResImpl)userroleres) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroleres.getClass(), false);
        copier.copy(clientModel, userroleres, null);
    }


    public void checkKey(IUserRoleRes userroleres){
        IUserRoleRes clientModel = userRoleResFeignClient.checkKey(userroleres.getUserRoleResId(),(UserRoleResImpl)userroleres) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroleres.getClass(), false);
        copier.copy(clientModel, userroleres, null);
    }


    public void update(IUserRoleRes userroleres){
        IUserRoleRes clientModel = userRoleResFeignClient.update(userroleres.getUserRoleResId(),(UserRoleResImpl)userroleres) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroleres.getClass(), false);
        copier.copy(clientModel, userroleres, null);
    }


    public void get(IUserRoleRes userroleres){
        IUserRoleRes clientModel = userRoleResFeignClient.get(userroleres.getUserRoleResId()) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroleres.getClass(), false);
        copier.copy(clientModel, userroleres, null);
    }


    public void create(IUserRoleRes userroleres){
        IUserRoleRes clientModel = userRoleResFeignClient.create((UserRoleResImpl)userroleres) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroleres.getClass(), false);
        copier.copy(clientModel, userroleres, null);
    }


    public void save(IUserRoleRes userroleres){
        IUserRoleRes clientModel = userRoleResFeignClient.save(userroleres.getUserRoleResId(),(UserRoleResImpl)userroleres) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroleres.getClass(), false);
        copier.copy(clientModel, userroleres, null);
    }


    public void remove(IUserRoleRes userroleres){
        userRoleResFeignClient.remove(userroleres.getUserRoleResId()) ;
    }


    public Page<IUserRoleRes> fetchDefault(SearchContext context){
        Page<UserRoleResImpl> page = this.userRoleResFeignClient.fetchDefault(context) ;
        return new PageImpl(page.getContent(),context.getPageable(),page.getTotalElements());
    }



}

