package cn.ibizlab.r7.runtime.client.base.service;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import cn.ibizlab.r7.runtime.core.client.model.IUserRoleType;
import cn.ibizlab.r7.runtime.client.base.config.BaseClientProperties;
import cn.ibizlab.r7.runtime.core.client.service.IUserRoleTypeClientService;
import cn.ibizlab.r7.runtime.client.base.model.UserRoleTypeImpl;
import cn.ibizlab.r7.runtime.client.base.feign.UserRoleTypeFeignClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;
import cn.ibizlab.r7.runtime.util.feign.FeignRequestInterceptor;
import cn.ibizlab.r7.runtime.util.SearchContext;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import cn.ibizlab.r7.runtime.util.feign.suport.SearchContextFeignEncode;
import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.codec.Decoder;
import feign.codec.Encoder;

/**
 * 实体[UserRoleType] 服务对象接口
 */
@Service
public class UserRoleTypeClientServiceImpl implements IUserRoleTypeClientService {

    UserRoleTypeFeignClient userRoleTypeFeignClient;

    @Autowired
    public UserRoleTypeClientServiceImpl(Decoder decoder, Encoder encoder, Client client, Contract contract ,FeignRequestInterceptor feignRequestInterceptor,BaseClientProperties baseClientProperties) {
        if (baseClientProperties.getServiceId()!=null) {
    		Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userRoleTypeFeignClient = nameBuilder.target(UserRoleTypeFeignClient.class,"http://"+baseClientProperties.getServiceId()+"/") ;
		}else if (baseClientProperties.getServiceUrl()!=null) {
			if (client instanceof LoadBalancerFeignClient) {
				client = ((LoadBalancerFeignClient) client).getDelegate();
			}
			Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(new SearchContextFeignEncode(encoder))
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.userRoleTypeFeignClient = nameBuilder.target(UserRoleTypeFeignClient.class,"http://"+baseClientProperties.getServiceUrl()+"/") ;
		}
    }
    @Override
    public IUserRoleType createModel() {
		return new UserRoleTypeImpl();
	}


    public void checkKey(IUserRoleType userroletype){
        IUserRoleType clientModel = userRoleTypeFeignClient.checkKey(userroletype.getUserRoleTypeId(),(UserRoleTypeImpl)userroletype) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroletype.getClass(), false);
        copier.copy(clientModel, userroletype, null);
    }


    public void getDraft(IUserRoleType userroletype){
        IUserRoleType clientModel = userRoleTypeFeignClient.getDraft(userroletype.getUserRoleTypeId(),(UserRoleTypeImpl)userroletype) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroletype.getClass(), false);
        copier.copy(clientModel, userroletype, null);
    }


    public void save(IUserRoleType userroletype){
        IUserRoleType clientModel = userRoleTypeFeignClient.save(userroletype.getUserRoleTypeId(),(UserRoleTypeImpl)userroletype) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroletype.getClass(), false);
        copier.copy(clientModel, userroletype, null);
    }


    public void create(IUserRoleType userroletype){
        IUserRoleType clientModel = userRoleTypeFeignClient.create((UserRoleTypeImpl)userroletype) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroletype.getClass(), false);
        copier.copy(clientModel, userroletype, null);
    }


    public void get(IUserRoleType userroletype){
        IUserRoleType clientModel = userRoleTypeFeignClient.get(userroletype.getUserRoleTypeId()) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroletype.getClass(), false);
        copier.copy(clientModel, userroletype, null);
    }


    public void remove(IUserRoleType userroletype){
        userRoleTypeFeignClient.remove(userroletype.getUserRoleTypeId()) ;
    }


    public void update(IUserRoleType userroletype){
        IUserRoleType clientModel = userRoleTypeFeignClient.update(userroletype.getUserRoleTypeId(),(UserRoleTypeImpl)userroletype) ;
        BeanCopier copier=BeanCopier.create(clientModel.getClass(), userroletype.getClass(), false);
        copier.copy(clientModel, userroletype, null);
    }


    public Page<IUserRoleType> fetchDefault(SearchContext context){
        Page<UserRoleTypeImpl> page = this.userRoleTypeFeignClient.fetchDefault(context) ;
        return new PageImpl(page.getContent(),context.getPageable(),page.getTotalElements());
    }



}

